#include <stdlib.h>
#include <string.h>
#include <kos.h>
#include "gui.h"
#include "menu.h"
#include "ws.h"

#define MENU_MAX_VISIBLE_ITEMS     12

typedef void (*menu_action_t)(char *arg);

typedef struct menu_item_s {

  char name[32];
  menu_action_t action;
  struct menu_item_s *next, *prev;
} menu_item_t;

char menuTitle[32];
menu_item_t *menuItemList=NULL;
int menuActive=0;
int menuChanged=0;
menu_action_t menuPrevious;

void menu_clear() {
  menu_item_t *lp;

  while(menuItemList != NULL) {
    lp = menuItemList->next;
    free(menuItemList);
    menuItemList = lp;
  }
}

void menu_addItem(const char *name, menu_action_t action) {
  menu_item_t *ni = malloc(sizeof(menu_item_t));

  if (strlen(name) > 31)
    return; //name too large, this should never happen.
  strncpy(ni->name, name, 32);
  ni->action = action;
  ni->next = menuItemList;
  ni->prev = NULL;
  if(menuItemList!=NULL) menuItemList->prev = ni;
  menuItemList = ni;
}

void menu_setTitle(const char *t) {
  strncpy(menuTitle, t, 32);
}

void drawMenu(int barPos, menu_item_t *mi) {
  int i;

  gui_begin();

  gui_drawBox(0xa0800000, 0x80800000, 110.0, 50.0, 530.0, 400.0, 50.0);
  gui_drawBox(0xa0800000, 0x80800000, 114.0, 54.0, 526.0, 90.0, 60.0);
  gui_printf(0xffffff00, 118.0, 58.0, 90.0, "%s", menuTitle);

  for(i=0; i<MENU_MAX_VISIBLE_ITEMS; i++, mi=mi->next) {
    if(mi==NULL)
      break;
    if(i==barPos) {
      gui_printf(0xff000000, 118.0, 94.0+i*24.0, 88.0, "%s", mi->name);
      gui_printf(0xffffffff, 122.0, 92.0+i*24.0, 90.0, "%s", mi->name);
    } else {
      gui_printf(0xffa0a0a0, 118.0, 94.0+i*24.0, 90.0, "%s", mi->name);
    }
  }

  gui_end();
}

void showMenu() {
  menu_item_t *firstItem = menuItemList;
  menu_item_t *selectedItem = menuItemList;
  int barPos = 0;
  uint8 dev;
  cont_cond_t cState;
  uint16 buttonSticky=0;

  menuActive = 1;
  menuChanged = 0;
  dev=maple_first_controller();

  cont_get_cond( dev, &cState );
  buttonSticky = ~(cState.buttons);

  while(menuActive) {
    cont_get_cond( dev, &cState );
    if(!(cState.buttons & CONT_DPAD_UP)) {
      if(!(buttonSticky & CONT_DPAD_UP)) {
	if(selectedItem->prev != NULL) {
	  if(barPos)
	    barPos--;
	  else
	    firstItem = firstItem->prev;
	  selectedItem = selectedItem->prev;
	}
      }
    }
    if(!(cState.buttons & CONT_DPAD_DOWN)) {
      if(!(buttonSticky & CONT_DPAD_DOWN)) {
	if(selectedItem->next != NULL) {
	  if(barPos < (MENU_MAX_VISIBLE_ITEMS-1))
	    barPos++;
	  else
	    firstItem = firstItem->next;
	  selectedItem = selectedItem->next;
	}
      }
    }
    if(!(cState.buttons & CONT_A)) {
      if(!(buttonSticky & CONT_A)) {
	if(selectedItem != NULL)
	  selectedItem->action(selectedItem->name);
	buttonSticky |= CONT_A;
      }
    }
    if(!(cState.buttons & CONT_B)) {
      if( !(buttonSticky & CONT_B)) {
	menuPrevious(NULL);
	buttonSticky |= CONT_B;
      } 
    } 
    if(menuChanged) {
      firstItem = selectedItem = menuItemList;
      barPos = 0;
      menuChanged = 0;
    }
    buttonSticky = ~(cState.buttons);
    drawMenu(barPos, firstItem);
  }
}

void quit(char *a) {
  ws_terminate = 1;
  menuActive = 0;
}

void reset(char *a) {
  ws_reset_game=1;
  menuActive = 0;
}

/* Each of the functions below loads a different menu (e.g. menu_main 
 * loads the main menu).
 * menu_none is a special case; this quits the entire menu system.
 */

void menu_none (char *);
void menu_loadRom (char *);
void menu_main (char *);

void menu_none ( char *a) {
  menuActive = 0;
}

static char curdir[4096];

void setRomPath(char *a) {
  strcpy(ws_rom_path, curdir);
  strcat(ws_rom_path, a);
  printf("Rom path: %s\n", ws_rom_path);
  menuActive = 0;
  ws_restart = 1;
}

void menu_loadRom(char *a) {
  file_t dir;
  dirent_t *de;

  if((a != NULL) && strcmp(a, "Load ROM")) {
    if(!strcmp(a, "../")) {
      *(strrchr(curdir, '/'))=0;
      *(strrchr(curdir, '/')+1)=0;
    } else {
      if(strcmp(a, "."))
	strcat(curdir, a);
    }
  }
  printf("menu_loadRom: curdir: %s\n", curdir);

  menu_clear();
  menu_setTitle("Load ROM");
  menuPrevious = menu_main;
  menuChanged = 1;

  dir = fs_open(curdir, O_RDONLY | O_DIR);
  if(!dir) {
    printf("Couldn't open %s\n", curdir);
    strcpy(curdir, "/");
    dir = fs_open(curdir, O_RDONLY | O_DIR);
  }
  while ((de = fs_readdir(dir))!=NULL) {
    if(*(de->name)!='.'){
      if(de->size == -1) {
	char name[32];
	strcpy(name, de->name);
	strcat(name, "/");
	printf("Read directory entry: %s\n", de->name);
	menu_addItem(name, menu_loadRom);
      } else {
	if(strstr(de->name, ".ws")!=NULL) {
	  printf("Read file entry: %s\n", de->name);
	  menu_addItem(de->name, setRomPath);
	}
      }
    }
  }
  if(strcmp(curdir, "/")) 
    menu_addItem("../", menu_loadRom);
  fs_close(dir);

}

void menu_main(char *a) {

  menu_clear();

  menu_setTitle("OSwan Menu");
  menu_addItem("Resume game", menu_none);
  menu_addItem("Reset game", reset);
  menu_addItem("Load ROM", menu_loadRom);
  menuPrevious = menu_none;
  menuChanged = 1;
}

void menu_open() {
  strcpy(curdir, ws_rom_path);
  
  if(strrchr(curdir, '/') != NULL) {
    *(strrchr(curdir, '/')+1)=0;
  }

  menu_main(NULL);
  showMenu();
}
